function [Rmin,OptM]=APIndexCoding(varargin)
%Finds near-optimal index code using rank minimization methods
%     APIndexCoding(M)
%     [Rmin,M]=APIndexCoding(M,tol)
%     [Rmin,M]=APIndexCoding(M,ALGO)
%     [Rmin,M]=APIndexCoding(M,b)
%     [Rmin,M]=APIndexCoding(M,tol,ALGO)
%     [Rmin,M]=APIndexCoding(M,tol,ALGO,b)
%
%       M is the matrix that represents the index coding problem. M is supposed to have the
%       following properties:
%         a) All the diagonal entries of M are equal to 1;
%         b) The (i,j)th entry of M is equal to 0 if user i does not have packet j;
%         c) The remaining entries should be set to NaN. These entries 
%            are usually represented by stars (*) in the literature.
%       tol is the degree of accuracy, that is, Rmin=rank(M,tol), tol=10^-3 in default.
%       ALGO is the algorithm used to minimize the rank.
%           'AP' uses Alternating Projections via eigenvalue decomposition;
%           'dirAP' represents directional Alternating Projections via eigenvalue decomposition;
%           'SVDAP' represents Alternating Projections via SVD;
%           'dirSVDAP' represents directional Alternating Projections via SVD;
%            For symmetric matrix M, ALGO='AP' in default, and for
%            non-symmetric matrix M, ALGO='dirSVDAP' in default.
%        b is the faction factor for vector linear
%
%     Output: optimal broadcast length Rmin and completed Matrix  low rank equal to Rmin.
%
%   Example: user 1 wants X1 Has X2, X3;
%            user 2 wants X2 Has X1, X3;
%            user 3 wants X3 Has X2, X4;         
%            user 4 wants X4 Has X1.     
%            Then for this example the input should be
%                 M=[ 1  NaN NaN  0 ;
%                    NaN  1  NaN  0 ;
%                     0  NaN  1  NaN;
%                    NaN  0   0   1 ;]
%            APIndexCoding(M) returns:
%              ans =
%                    2
%

%   Copyright 2015, Xiao Huang and Salim El Rouayheb, CSI Lab.
%   $Revision: 1.1.0 $  $Date: 2015/10/26 18:00:00 $


%
[n,m,OneIndex,ZeroIndex,tol,ALGO]=getVals(varargin{:});% Get Values

% Initialization
Rmin=n;% Set the initial guess of Rmin here
Rnk=n; % Current optimal Index Code Length

% Find the optimal Matrix OptM
[exi,OutM]=AltPrj(Rmin,ALGO);% Test whether exist such M with rank Rmin by using ALGO
while(exi) % If exist such M with rank Rmin
    Rmin=Rmin-1;
    OptM=OutM; % Output optimal M with low rank
    [exi,OutM]=AltPrj(Rmin,ALGO);% Test whether exist such M with rank Rmin by using ALGO
end
Rmin=Rmin+1; % Optimal Index Code Length
Rmin=Rmin/b;
%
%--------------------------------------------------------------------------







%-------------------Alternating Projection Function-------------------%
    function [exi,OutM]=AltPrj(r,ALGO)
        % Initializations
        Ite=0; % Iteration Number
        % check if the SVD is reducing the rank by increasing the singular
        % values to infinite
        CheckInf=0; 
        DisM=1;
        switch ALGO
      %------Algorithm APIndexCoding via eigenvalue decomposition
            case{'AP'}
                M=ones(n); % AP start point (in region D)
                M(ZeroIndex)=0;
                OutM=M;
                while (Rnk>r && DisM>tol/n^3) % Check stopping criteria
                    % Projection on region C
                    [V,D]=eig(M); % Eigenvalue decomposition
                    eigM=diag(D); % Eigenvalues of M
                    eigM(eigM<0)=0; % Only want positive semi-definite matrix
                    [~,order] = sort(eigM); % Sort eigenvalues in ascending order
                    D(:,order(1:n-r))=0;
                    M=V*D*V';
                    DisM=norm(OutM-M); % Check how much change in M
                    OutM=M;
                    % Projection on region D
                    M(ZeroIndex)=0;
                    M(OneIndex)=1;
                    Rnk=rank(M,tol); % Current optimal Index Code Length
                end
       %-----Algorithm Directional APIndexCoding via eigenvalue decomposition
            case{'dirAP'} 
                M=ones(n); % AP start point (in region D)
                M(ZeroIndex)=0;
                OutM=M;
                tmpN=zeros(1,60);
                while (Rnk>r && DisM>tol/n^3)
                    Ite=1+Ite; % Iteration Number
                    % Projection on region C
                    [V,D]=eig(M); % Eigenvalue decomposition
                    eigM=diag(D); % Eigenvalues of M
                    eigM(eigM<0)=0; % Only want positive semi-definite matrix
                    [~,order] = sort(eigM);  % Sort eigenvalues in ascending order
                    D(:,order(1:n-r))=0;
                    M=V*D*V'; % First piont (in region C)
                    OutM=M; % This M maybe the optimal solution
                    % Projection on region D
                    M(ZeroIndex)=0; 
                    M(OneIndex)=1;
                    Rnk=rank(M,tol); % Current optimal Index Code Length
                    % Directional Projection
                    M2=M;  % Second piont (in region D)
                    [V,D]=eig(M); % Eigenvalue decomposition
                    eigM=diag(D); % Eigenvalues of M
                    eigM(eigM<0)=0; % Only want positive semi-definite matrix
                    [~,order] = sort(eigM);  % Sort eigenvalues in ascending order
                    D(:,order(1:n-r))=0;
                    M=V*D*V';  % Third piont (in region C)
                    DisM=norm(OutM-M);
                    NormF=norm(OutM-M2,'fro'); % Frobenius distance of first piont and second piont
                    M=OutM+NormF^2/trace((OutM-M)'*(OutM-M2))*(M-OutM); % Fourth piont
                    % Check stopping criteria every 60 iterations 
                    NumN=mod(Ite-1,60)+1;
                    tmpN(NumN)=NormF;
                    if NumN==60
                        ENormF=sum(tmpN)/60; % Average Frobenius distance in 60 interations
                        if Ite>60 && (tmpF<ENormF+tol/n)
                            break;
                        end
                        tmpF=ENormF;
                    end
                    % Projection on region D
                    M(ZeroIndex)=0; 
                    M(OneIndex)=1;
                end
      %------Algorithm APIndexCoding via Singular value decomposition    
            case{'SVDAP'}
                M=rand(n,m); % Randomly pick SVDAP start point
                M(ZeroIndex)=0;
                M(OneIndex)=1;
                OutM=M;
                while (Rnk>r && DisM>tol/n^3 && CheckInf<n-Rnk+5)
                    Ite=Ite+1;
                    % Projection on region C
                    [U,S,V]=svd(M);
                    Rnk=length(find(S>tol)); % Current optimal Index Code Length
                    S(r+1:n,:)=0;
                    M=U*S*V';
                    CheckInf=S(1);
                    DisM=norm(OutM-M);
                    OutM=M;
                    % Projection on region D
                    M(ZeroIndex)=0;
                    M(OneIndex)=1;
                end
      %------Algorithm directional APIndexCoding via Singular value decomposition 
            case{'dirSVDAP'}
                M=rand(n,m);% Randomly pick SVDAP start point
                M(ZeroIndex)=0;
                M(OneIndex)=1;
                OutM=M;
                tmpN=zeros(1,60);
                while (Rnk>r && DisM>tol/n^3  && CheckInf<n-Rnk+5)
                    Ite=1+Ite; % Iteration Number
                    % Projection on region C
                    [U,S,V]=svd(M);
                    S(r+1:n,:)=0;
                    M=U*S*V'; % First piont (in region C)
                    DisM=norm(OutM-M);
                    OutM=M; % This M maybe the optimal solution
                    % Projection on region D
                    M(ZeroIndex)=0; 
                    M(OneIndex)=1;
                    M2=M; % Second piont (in region D)
                    [U,S,V]=svd(M);
                    Rnk=length(find(S>tol));
                    S(r+1:n,:)=0;
                    CheckInf=S(1);
                    M=U*S*V';  % Third piont (in region C)
                    NormF=norm(OutM-M2,'fro'); % Frobenius distance of first piont and second piont
                    M=OutM+NormF^2/trace((OutM-M)'*(OutM-M2))*(M-OutM); % Fourth piont
                    % Check stopping criteria every 60 iterations 
                    NumN=mod(Ite-1,60)+1;
                    tmpN(NumN)=NormF;
                    if NumN==60
                        ENormF=sum(tmpN)/60; % Average Frobenius distance in 60 interations
                        if Ite>60 && (tmpF<ENormF || abs(tmpF-ENormF)<tol/n)
                            DisM=0;
                        end
                        tmpF=ENormF;
                    end
                    % Projection on region D
                    M(ZeroIndex)=0;
                    M(OneIndex)=1;
                end
        end
        exi=Rnk<=r & CheckInf<n-Rnk+5;
    end
%--------------------------------------------------------------------------



%----------------------------getVals Function --------------------------
    function [n,m,OneIndex,ZeroIndex,tol,ALGO]=getVals(varargin)
        % Default Settings
        M=[]; % Initialization of the Matrix
        tol=0.1^3;
        ALGO='dirAP';
        b=1;
        n=0;
        m=0;
        OneIndex=[];
        NumVal=length(varargin); % Check the number of Input Values
        if NumVal==0 % No Input
            warning('The input is empty!')
        elseif NumVal>=1 % One Input, then it must be the Matrix
            M=varargin{1};
            [n,m]=size(M);
            OneIndex=1:n+1:n^2;% Indices of nonzero elements. Need to set to 1
            ZeroIndex=M==0;
            if ~isequal(ZeroIndex,ZeroIndex') % Check whether Input Matrix symmetric
                ALGO='dirSVDAP';
            end
            if NumVal>=2 % Two Input Values
                if ischar(varargin{2})
                    ALGO=varargin{2};
                else
                    if varargin{2}<1
                        tol=varargin{2}; % Degree of accuracy
                    else
                        b=varargin{2};
                    end
                end
                if NumVal>=3
                    switch varargin{3}
                        case{'AP','dirAP'}
                            % Make Input Matrix symmetric
                            ZeroIndex=bitor(ZeroIndex,ZeroIndex');
                            M(ZeroIndex)=0;
                            ALGO=varargin{3};
                        case{'SVDAP','dirSVDAP'}
                            ALGO=varargin{3};
                        otherwise
                            warning('Wrong Algorithm! Change to default setting now.')
                    end
                    if NumVal>=4
                        b=varargin{4}; % Fraction factor
                        if NumVal>=5
                            warning('Too many Input!')
                        end
                    end
                end
            end
            if n>m
                OneIndex=M==1;%indices of nonzero elements. Need to set to 1
            elseif n<m
                error('Please input a square matrix or a "tall" matrix (m>n).')
            end 
        end
        if b>=2
           if n~=m
               error('Fraction only for square Matrix!')
           else
               tmpU=zeros(b*n,n);
               tmpV=zeros(n,b*n);
               for tmpi=1:n
                   tmpU(b*tmpi-b+1:b*tmpi,tmpi)=1;
                   tmpV(tmpi,b*tmpi-b+1:b*tmpi)=1;
               end
               tmpM=~ZeroIndex;
               tmpM(OneIndex)=0;
               M=tmpU*tmpM*tmpV;
               n=b*n;
               m=b*m;
               OneIndex=1:n+1:n^2;
               M(OneIndex)=1;
           end
        end
        ZeroIndex=M==0;% Set Positions of zeros
    end
%--------------------------------------------------------------------------

end