function FindNetworkCode(varargin)
%Finds a linear network code using rank minimization methods
%     FindNetworkCode(Edges,Demand)
%     FindNetworkCode(Edges,Demand,tol)
%
%       Edges is the edges of the network, it indicates the direction of the edge;
%         If we have an edge going from node i to j, then add row [i j;] into Edges.
%       Demand is a vector indicating the destinations and the corresponding sources;
%         It assigns number to each node:
%           a) The number of the corresponding source if it is a receiver,
%           b) 0 otherwise.
%       tol is the degree of accuracy, that is, Rmin=rank(M,tol), tol=10^-3 in default
%  
%     Output: matrix M indicating the local encoding functions. The rows of
%       M are the edges of the networks followed by the destinations. Its 
%       columns are also the edges followed by the data packets. The edges 
%       are ordered in the same manner of the Network vector, then the data 
%       packets and destinations are oredered according to the demand vector
%       from the first non zero value to the last one.
%
%
%In other words, the matrix indicates on row i what the ith edge should send
%as combination of its incoming edges and the source packets.
%
%   Example: the butterfly network ("Butterfly_net.pdf")
%        In this case, Edges=[1 5;1 3;2 3;2 6;3 4;4 5;4 6].
%        Since node 5 needs the packet in node 2 and node 6 needs the packet
%        in node 1, Demand=[0 0 0 0 2 1];
%        The output matrix OptM will be indexed as follows:
%             M[i,j] is the message on edge[i,j];
%             D[k] is the decoding message on node k;
%             M[1,3] = 0.3688*X1 ;
%             M[1,5] = 0.9657*X1 ;
%             M[2,3] = -0.6845*X2 ;
%             M[2,6] = 1.3414*X2 ;
%             M[3,4] = 1.6315*M[1,3] + 0.7577*M[2,3] ;
%             M[4,5] = -1.1880*M[3,4] ;
%             M[4,6] = 1.3503*M[3,4]  ;
%             D[5] = 1.1944*M[1,5] +   1.6153*M[4,5] ;
%             D[6] = 0.6453*M[2,6] +   1.2311*M[4,6] ;

%   Copyright 2015, Xiao Huang, Rawad Bitar and Salim El Rouayheb, CSI Lab.
%   $Revision: 2.0.2 $  $Date: 2015/10/06 16:14:42 $

%
[Edges,Demand,tol]=getVals(varargin{:});
[NetMat,OptL,SourceIndex,DemNonZero]=CreateM(Edges,Demand); % Get Network Matrix and number of edges
% Use LDG to find the optimal solution

[NewMat,RemainRow,IndexMat]=DecMatSize(NetMat);

IfLDG=1;
Newn=length(NewMat);

NewOptL=OptL-n+Newn;
[Rmin,OptM]=LDG(NewMat);
% Use APIndexCoding to find the optimal solution
for APIter=1:5 % Try APIndexCoding again for at most 5 times
    if Rmin>NewOptL  && NewOptL>0 % Can't find the optimal solution
        IfLDG=0;
        [Rmin,OptM]=APIndexCoding(NewMat,tol);
    else
        break;
    end
end



tmpOptM=isnan(NetMat)*1;
ReRowNum=length(RemainRow);
for Remaini=1:ReRowNum
    tmpRow=zeros(1,n);
    for Remainj=1:ReRowNum
        if Remainj~=Remaini
            tmpRow(IndexMat(Remaini,Remainj))=OptM(Remaini,Remainj);
        end
    end
    tmpOptM(RemainRow(Remaini),:)=tmpRow;
end

AllzeroRow0=[];
AllzeroRow1=find(sum(tmpOptM,2)==0);
while(~isequal(AllzeroRow0,AllzeroRow1))
AllzeroRow1=find(sum(tmpOptM,2)==0);
tmpOptM(:,AllzeroRow1)=zeros(n,length(AllzeroRow1));
AllzeroRow0=find(sum(tmpOptM,2)==0);
end




OptM=tmpOptM;
%OptM(NewMat==1)=0;


if Rmin>NewOptL  && NewOptL>0% Can't find the optimal solution
    fprintf('Cannot find the scalar linear network code.\n');
else
    if IfLDG==1
        fprintf('Field is GF(2);\n')
        fprintf('M[i,j] is the message on edge[i,j];\nD[k] is the decoding message on node k;\n');
        %PrintMat=isnan(NetMat);
        PrintMat=OptM;
        TerminalNum=length(DemNonZero(1,:)); % Number of terminals
        for printi=1:OptL+TerminalNum
            if printi<=OptL % OptL equals to number of edges
                fprintf('M[%d,%d] = ',Edges(printi,:));
            else
                fprintf('D[%d] = ',DemNonZero(2,printi-OptL));
            end

            IndexInputs=find(PrintMat(printi,:));
            for printj=1:length(IndexInputs)
                IndexInputi=IndexInputs(printj);
                if  printj<length(IndexInputs) && IndexInputi<=OptL;
                    fprintf('M[%d,%d] + ',Edges(IndexInputi,:));
                elseif IndexInputi<=OptL;
                    fprintf('M[%d,%d]',Edges(IndexInputi,:));
                else
                    fprintf('X%d',SourceIndex(IndexInputi-OptL));
                end        

            end
            fprintf(' ;\n');
        end

    else
        fprintf('Field is R;\n') 
        fprintf('M[i,j] is the message on edge[i,j];\nD[k] is the decoding message on node k;\n');
        PrintMat=OptM;
        TerminalNum=length(DemNonZero(1,:)); % Number of terminals
        for printi=1:OptL+TerminalNum
            if printi<=OptL % OptL equals to number of edges
                fprintf('M[%d,%d] = ',Edges(printi,:));
            else
                fprintf('D[%d] = ',DemNonZero(2,printi-OptL));
            end

            IndexInputs=find(PrintMat(printi,:));
            for printj=1:length(IndexInputs)
                IndexInputi=IndexInputs(printj);
                if  printj<length(IndexInputs) && IndexInputi<=OptL 
                    fprintf('%7.4f*M[%d,%d] + ',OptM(printi,IndexInputi),Edges(IndexInputi,:));
                elseif IndexInputi<=OptL
                    fprintf('%7.4f*M[%d,%d]',OptM(printi,IndexInputi),Edges(IndexInputi,:));
                else
                    fprintf('%7.4f*X%d',OptM(printi,IndexInputi),SourceIndex(IndexInputi-OptL));
                end        

            end
            fprintf(' ;\n');
        end
    end
end





%
%-------------------------------------------------------------------------









%-------------- Function Create Matrix for the Network--------------------
    function[NetMat,OptL,SourceIndex,DemNonZero]=CreateM(Edges,Demand)     
        OptL=length(Edges); % Count number of Edeges 
        Demand=[Demand;1:length(Demand)]; % Add Index to the Demand
        DemNonZero=Demand(:,Demand(1,:)~=0); % Nonzero items in Demand
        SourceIndex=DemNonZero(1,:); % Get Source Index
        SourceNum=length(SourceIndex); % Number of sources
        TerminalNum=length(DemNonZero(1,:)); % Number of demands
        n=OptL+SourceNum; % Dimension of the Matrix 
        % Begin to create the Network Matrix NetMat
        NetMat=[eye(OptL,n);zeros(TerminalNum,n)]; % Initialization
        for i=1:OptL
            % The first edge is from Node i to j
            Nodei=Edges(i,1);
            InputEdge= Edges(:,2)==Nodei; % Edges to Node i
            NetMat(i,InputEdge)=nan;
            NetMat(i,OptL+find(SourceIndex==Nodei))=nan;
        end
        for i=OptL+1:OptL+TerminalNum
            % demand i request Source j
            demandi=i-OptL; % demand Index
            Sourcej=find(SourceIndex==DemNonZero(1,demandi));
            NetMat(i,OptL+Sourcej)=1; % Set 1 means request
            % Node i requests demand i
            Nodei=DemNonZero(2,demandi);
            InputEdge= Edges(:,2)==Nodei; % Edges to Node i
            NetMat(i,InputEdge)=nan;
        end
    end
%--------------------------------------------------------------------------




%-------------------------Decrease Matrix Size-----------------------------
    function [NewMat,RemainRow,IndexMat]=DecMatSize(NetMat)
        Indexi=1;
        NewMat=NetMat;
        tmpIndex=1:n;
        IndexMat=repmat(tmpIndex,n,1);
        RemainRow=find(sum(NewMat==0,2)~=n-2);
        RemoveIndex=find(sum(isnan(NewMat),2)==1);
        RemoveRowi=find(sum(isnan(NewMat),2)==1,1);
        while(~isempty(RemoveRowi))
            Hasi= isnan(NewMat(RemoveRowi,:));
            Wanti= isnan(NewMat(:,RemoveRowi));
            NewMat(Wanti,Hasi)=nan;
            IndexMat(Wanti,Hasi)=RemoveIndex(Indexi);
            NewMat(RemoveRowi,:)=[];
            NewMat(:,RemoveRowi)=[];
            IndexMat(RemoveRowi,:)=[];
            IndexMat(:,RemoveRowi)=[];
            RemoveRowi=find(sum(isnan(NewMat),2)==1,1);
            Indexi=Indexi+1;
        end

        tmpNot1=diag(NewMat)~=1;
        NewMat(tmpNot1,:)=[];
        NewMat(:,tmpNot1)=[];
        IndexMat(tmpNot1,:)=[];
        IndexMat(:,tmpNot1)=[];
        RemainRow(tmpNot1)=[];
    end


%--------------------------------------------------------------------------


%----------The Least Difference Greedy (LDG) Clique-Cover Algorithm-------
    function [Rmin,OptM]=LDG(NetMat)
        NetMat(isnan(NetMat))=9; % Change NaN (star) to 9
        % Initializations
        rowi=1; % Iteration pointer
        [ni,m]=size(NetMat);
        RN=ni; % Number of rows remaining
        RowIndex=1:ni; % Record row index
        while(RN>rowi)
            Mri=abs(NetMat(rowi+1:RN,:)-repmat(NetMat(rowi,:),RN-rowi,1)); % Distance matrix for row i
            Mri(Mri==1)=inf; % Distance between 0 and 1 is Infinity
            Mri(Mri==8|Mri==9)=1;
            [DistOrd,Rj]=sort(sum(Mri,2)); % Distance between row ri and others
            if DistOrd(1)==inf % Row ri can not be merged 
                rowi=rowi+1;
            else
                rowj=Rj(randi([1,sum(isfinite(DistOrd))]))+rowi; % Random pick a mergeable row
                % Merge ri and rj 
                for ck=1:m  % Row merging
                    if NetMat(rowi,ck)==9
                        NetMat(rowi,ck)=NetMat(rowj,ck);
                    end
                end
                NetMat(rowj,:)=[]; % Delete row rj
                RowIndex(RowIndex==rowj)=rowi;
                RowIndex(RowIndex>rowj)=RowIndex(RowIndex>rowj)-1;
                RN=RN-1; % Number of rows remaining
            end
        end
        Rmin=min(size(NetMat));
        NetMat(NetMat==9)=0; % Change remaining NaN (star) to 0
        OptM(1:ni,:)=NetMat(RowIndex,:); % The optimal completed Matrix
    end
%--------------------------------------------------------------------------





%----------------------------getVals Function --------------------------
    function [Edges,Demand,tol]=getVals(varargin)
        % Default Settings
        Edges=[];
        Demand=[];
        tol=10^-3; % Degree of accuracy
        NumVal=length(varargin); % Check the number of Input Values
        if NumVal<2 % Not enough Input
            warning('The input is not enough!')
        elseif NumVal>=2
                Edges=varargin{1};
                Edges=unique(Edges,'rows');
                Demand=varargin{2};
                if length(Demand)~=max(max(Edges))
                    error('The Demand vector does not match the given Network');
                end        
                if NumVal>=3
                    tol=varargin{3};
                    if NumVal>=4
                        warning('Too many Inputs!')
                    end
                end
        end
    end
%--------------------------------------------------------------------------




end


%---------------------------------Examples---------------------------------
% Example2_Network=[1 6;1 7;1 4;2 6;2 4;2 8;3 4;3 7;3 8;4 5;5 12;5 13;5
% 14;6 9;7 10;8 11;9 12;10 13;11 14];
%Demand=[zeros(1,11) 3 2 1];
%FindNetworkCode(Example2_Network,Demand)









%--------------------------------------------------------------------------

